class Balloon < ActiveRecord::Base
  belongs_to :speech_balloon
  belongs_to :system_picture
  
  validates :speech_balloon_id, :numericality => {:allow_blank => true}
  validates :system_picture_id, :presence => true, :numericality => true, :existence => {:both => false}
  validates :x, :presence => true, :numericality => true
  validates :y, :presence => true, :numericality => true
  validates :width, :presence => true, :numericality => true, :natural_number => true
  validates :height, :presence => true, :numericality => true, :natural_number => true
  validates :r, :presence => true, :numericality => true
#  validates :caption, :presence => true
#  validates :settings, :presence => true

  before_validation :valid_encode
  
  def valid_encode
    ['settings'].each do |a|
      next if attributes[a] == nil
      raise Pettanr::BadRequest unless attributes[a].valid_encoding?
    end
  end
  
  def url
    '/system_pictures/' + self.system_picture.filename
  end
  
  def visible? roles
    if MagicNumber['run_mode'] == 0
      return false unless guest_role_check(roles)
    else
      return false unless reader_role_check(roles)
    end
    return true if self.speech_balloon.panel.own?(roles)
    self.speech_balloon.panel.visible? roles
  end
  
  def supply_default
    self.x = 0
    self.y = 0
    self.width = 100
    self.height = 100
    self.r = 0
self.system_picture_id = 1
  end
  
  def get_speech_balloon
    self.speech_balloon
  end
  
  def get_panel_id
    if self.get_speech_balloon == nil or self.get_speech_balloon.panel == nil or self.get_speech_balloon.panel.new_record?
      0
    else
      self.get_speech_balloon.panel.id
    end
  end
  
  def tag_id c = nil
    'panel' + tag_panel_id + tag_element_type + tag_element_id + tag_element_part_type + tag_element_part_id + c.to_s
  end
  
  def field_tag_id f
    self.tag_id + f.to_s
  end
  
  def tag_panel_id
    self.get_panel_id.to_s
  end
  
  def tag_element_id
    r = if self.get_speech_balloon == nil or self.get_speech_balloon.new_record?
      0
    else
      self.get_speech_balloon.id
    end
    r.to_s
  end
  
  def tag_element_type
    'speech_balloon'
  end
  
  def tag_element_part_id
    self.new_record? ? '0' : self.id.to_s
  end
  
  def tag_element_part_type
    'balloon'
  end
  
  def field_tree f
    'panels-' + self.tag_panel_id + '-speech_balloons_attributes-' + self.tag_element_id + '-balloon_attributes-' + f.to_s
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.list page = 1, page_size = self.default_page_size
    opt = {}
    opt.merge!(Balloon.list_opt)
    opt.merge!({:limit => page_size, :offset => (page -1) * page_size}) if page_size > 0
    opt.merge!({:conditions => 'panels.publish > 0', :order => 'balloons.updated_at desc'})
    Balloon.find(:all, opt)
  end
  
  def self.list_opt
    {:include => {:speech_balloon => {:panel => {:author => {}}, :speech => {}, :speech_balloon_template => {} }}}
  end
  
  def self.list_json_opt
    {:include => {:speech_balloon => {:include => {:panel => {:include => {:author => {} }}, :speech => {}, :speech_balloon_template => {} }}}}
  end
  
  def self.show cid, au
    opt = {}
    opt.merge!(Balloon.show_opt)
    res = Balloon.find(cid, opt)
    raise ActiveRecord::Forbidden unless res.visible?(au)
    res
  end
  
  def self.show_opt
    {:include => {:speech_balloon => {:panel => {:author => {}}, :speech => {}, :speech_balloon_template => {} }}}
  end
  
  def self.show_json_opt
    {:include => {:speech_balloon => {:include => {:panel => {:include => {:author => {} }}, :speech => {}, :speech_balloon_template => {} }}}}
  end
  
  def scenario
  end
  
  def plain_scenario
  end
  
end
